'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQLRawError = undefined;
exports.canDisplayGraphiQL = canDisplayGraphiQL;
exports.handleResult = handleResult;
exports.handleError = handleError;
exports.graphqlError = graphqlError;
exports.exec = exec;

var _graphql = require('graphql');

var _accepts = require('accepts');

var _accepts2 = _interopRequireDefault(_accepts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
/**
 *  Fork Copyright (c) 2016, Martin Heidegger,
 *
 *  Original Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */

var GraphQLRawError = exports.GraphQLRawError = function GraphQLRawError(status, errors) {
  _classCallCheck(this, GraphQLRawError);

  this.status = status;
  this.errors = errors;
};

/**
 * Helper function to determine if GraphiQL can be displayed.
 */


function canDisplayGraphiQL(request, params) {
  // If `raw` exists, GraphiQL mode is not enabled.
  // Allowed to show GraphiQL if not requested as raw and this request
  // prefers HTML over JSON.
  return !params.raw && (0, _accepts2.default)(request).types(['json', 'html']) === 'html';
}

function handleResult(formatErrorFn, response, result) {
  // If no data was included in the result, that indicates a runtime query
  // error, indicate as such with a generic status code.
  // Note: Information about the error itself will still be contained in
  // the resulting JSON payload.
  // http://facebook.github.io/graphql/#sec-Data
  if (result && result.data === null) {
    response.statusCode = 500;
  }
  // Format any encountered errors.
  if (result && result.errors) {
    result.errors = result.errors.map(formatErrorFn || _graphql.formatError);
  }

  return result;
}

function handleError(response, error) {
  // If an error was caught, report the httpError status, or 500.
  response.statusCode = error.status || 500;

  if (error instanceof GraphQLRawError) {
    return { errors: error.errors };
  }
  return { errors: [error] };
}

function graphqlError(status, errors) {
  return new GraphQLRawError(status, errors);
}

function exec(schema, rootValue, context, extensionsInput, documentAST, variables, operationName) {
  var executor = void 0;
  try {
    executor = (0, _graphql.execute)(schema, documentAST, rootValue, context, variables, operationName);
  } catch (contextError) {
    return Promise.reject(graphqlError(400, [contextError]));
  }
  if (typeof extensionsInput === 'function') {
    (function () {
      var extensionsFn = extensionsInput;
      executor.then(function (result) {
        return Promise.resolve(extensionsFn({
          document: documentAST,
          variables: variables,
          operationName: operationName,
          result: result
        })).then(function (extensions) {
          result.extensions = extensions;
          return result;
        });
      });
    })();
  }
  return executor;
}