'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (req, charset) {
  return new Promise(function (resolve, reject) {
    // Assert charset encoding per JSON RFC 7159 sec 8.1
    if (charset.slice(0, 4) !== 'utf-') {
      throw (0, _httpErrors2.default)(415, 'Unsupported charset "' + charset.toUpperCase() + '".');
    }

    // Get content-encoding (e.g. gzip)
    var contentEncoding = req.headers['content-encoding'];
    var encoding = typeof contentEncoding === 'string' ? contentEncoding.toLowerCase() : 'identity';
    var length = encoding === 'identity' ? req.headers['content-length'] : null;
    var limit = 100 * 1024; // 100kb
    var stream = decompressed(req, encoding);

    // Read body from stream.
    (0, _rawBody2.default)(stream, { encoding: charset, length: length, limit: limit }, function (err, body) {
      if (err) {
        return reject(err.type === 'encoding.unsupported' ? (0, _httpErrors2.default)(415, 'Unsupported charset "' + charset.toUpperCase() + '".') : (0, _httpErrors2.default)(400, 'Invalid body: ' + err.message + '.'));
      }

      resolve(body);
    });
  });
};

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _rawBody = require('raw-body');

var _rawBody2 = _interopRequireDefault(_rawBody);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Return a decompressed stream, given an encoding.
function decompressed(req, encoding) {
  switch (encoding) {
    case 'identity':
      return req;
    case 'deflate':
      return req.pipe(_zlib2.default.createInflate());
    case 'gzip':
      return req.pipe(_zlib2.default.createGunzip());
  }
  throw (0, _httpErrors2.default)(415, 'Unsupported content-encoding "' + encoding + '".');
}

// Read and parse a request body.

/**
 *  Fork Copyright (c) 2016, Martin Heidegger,
 *
 *  Original Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */